/*---------------------------------------------------------------------\
|                                                                      |
|                      __   __    ____ _____ ____                      |
|                      \ \ / /_ _/ ___|_   _|___ \                     |
|                       \ V / _` \___ \ | |   __) |                    |
|                        | | (_| |___) || |  / __/                     |
|                        |_|\__,_|____/ |_| |_____|                    |
|                                                                      |
|                               core system                            |
|                                                        (C) SuSE GmbH |
\----------------------------------------------------------------------/

  File:       PMYouPatchInfo.h

  Author:     Cornelius Schumacher <cschum@suse.de>
  Maintainer: Cornelius Schumacher <cschum@suse.de>

  Purpose: Utility class for reading the patch information

/-*/
#ifndef PMYouPatchInfo_h
#define PMYouPatchInfo_h

#include <list>
#include <string>
#include <map>

#include <y2util/Url.h>
#include <y2util/Pathname.h>
#include <y2util/LangCode.h>
#include <y2util/FSize.h>

#include <y2pm/PMError.h>
#include <y2pm/MediaAccess.h>

#include <y2pm/PMYouPatchPtr.h>
#include <y2pm/PMYouPackageDataProviderPtr.h>
#include <y2pm/PMYouPatchTags.h>
#include <y2pm/PMYouSettings.h>

#include <y2pm/PMYouPatchInfoPtr.h>

///////////////////////////////////////////////////////////////////
//
//	CLASS NAME : PMYouPatchInfo
/**
 * Patch information.
 **/
class PMYouPatchInfo : public CountedRep {
  REP_BODY(PMYouPatchInfo);

  public:
    /**
      Constructor

      @param settings  Pointer to object representing the YOU settings
    */
    PMYouPatchInfo( PMYouSettingsPtr settings );

    /**
     * Destructor
     **/
    ~PMYouPatchInfo();

    /**
      Get directory file listing all available patches.
    */
    PMError getDirectory( bool useMediaDir );

    /**
     * Get patches from specified location.
     *
     * @param paths     Object holding the path location information.
     * @param patches   List of patch objects where the results are stored.
     * @param reload    If true, reload patch files from server.
     * @param checkSig  If true, check GPG signature of patch info files.
     **/
    PMError getPatches( std::vector<PMYouPatchPtr> &patches );
    
    /**
      Read patch info files from directory.

      @param baseUrl          Base of URL where patches are located.
      @param patches          List of patch objects where the results are stored.
      @param checkSig         If true, check GPG signature of patch info files.
      @param processMediaDir  If true, read patch file from media directory.
    **/
    PMError readDir( std::vector<PMYouPatchPtr> &patches,
                     bool useMediaDir = true );

    /**
      Read patch info from file.
     
      @param path     File path of package info file.
      @param fileName Name of patch file.
      @param patch    Patch object where the results are stored.
    */
    PMError readFile( const Pathname &path, const std::string &fileName,
                      PMYouPatchPtr &patch );

    /**
     * Parse package info.
     *
     * @param packages String containing the package information.
     * @param patch    Patch the packages belong to.
     **/
    PMError parsePackages( const std::string &packages,
                           const PMYouPatchPtr &patch );

    /**
      Parse Files tag.
    */
    PMError parseFiles( const std::string &files, const PMYouPatchPtr &patch );

    PMYouPackageDataProviderPtr packageDataProvider() const;

    PMError readDirectoryFile( const Pathname &,
                               std::list<std::string> &patchFiles );

    PMError processMediaDir();

    int mediaNumber( const PMYouPatchPtr & );

  protected:
    std::string tagValueLocale ( YOUPatchTagSet::Tags tagIndex,
                                 std::istream &input );

    std::string tagValue( YOUPatchTagSet::Tags tagIndex, std::istream &input,
                          const std::string &locale = "" );
    std::string tagMultiValue( YOUPatchTagSet::Tags tagIndex, 
                               std::istream& input );

    std::string tagValue( YOUPackageTagSet::Tags tag );

    PMError createPackage( const PMYouPatchPtr &patch );

    void readMediaMap( const Pathname &file );

  private:
    YOUPatchTagSet _patchTagSet;
    YOUPackageTagSet _packageTagSet;
    
    PMYouSettingsPtr _settings;

    MediaAccess _media;

    PMYouPackageDataProviderPtr _packageDataProvider;
    
    bool _doneMediaDir;
    bool _doneDirectory;

    int _totalPatchFileCount;

    std::map<std::string,int> _mediaMap;
};

///////////////////////////////////////////////////////////////////

#endif // PMYouPatchInfo_h
